@@ -1,7 +1,7 @@
 ;;; tramp-smb.el --- Tramp access functions for SMB servers
 
-;; Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007,
-;;   2008 Free Software Foundation, Inc.
+;; Copyright (C) 2002, 2003, 2004, 2005, 2006, 2007, 2008,
+;;   2009 Free Software Foundation, Inc.
 
 ;; Author: Michael Albinus <michael.albinus@gmx.de>
 ;; Keywords: comm, processes
@@ -44,7 +44,7 @@
 ;; Add a default for `tramp-default-method-alist'. Rule: If there is
 ;; a domain in USER, it must be the SMB method.
 (add-to-list 'tramp-default-method-alist
-	     `(nil "%" ,tramp-smb-method))
+	     `(nil ,tramp-prefix-domain-regexp ,tramp-smb-method))
 
 ;; Add a default for `tramp-default-user-alist'. Rule: For the SMB method,
 ;; the anonymous user is chosen.
@@ -139,7 +139,7 @@ See `tramp-actions-before-shell' for more info.")
     (directory-files-and-attributes . tramp-smb-handle-directory-files-and-attributes)
     (dired-call-process . ignore)
     (dired-compress-file . ignore)
-    ;; `dired-uncache' performed by default handler
+    (dired-uncache . tramp-handle-dired-uncache)
     ;; `expand-file-name' not necessary because we cannot expand "~/"
     (file-accessible-directory-p . tramp-smb-handle-file-directory-p)
     (file-attributes . tramp-smb-handle-file-attributes)
@@ -150,7 +150,7 @@ See `tramp-actions-before-shell' for more info.")
     (file-remote-p . tramp-handle-file-remote-p)
     (file-modes . tramp-handle-file-modes)
     (file-name-all-completions . tramp-smb-handle-file-name-all-completions)
-    ;; `file-name-as-directory' performed by default handler
+    (file-name-as-directory . tramp-handle-file-name-as-directory)
     (file-name-completion . tramp-handle-file-name-completion)
     (file-name-directory . tramp-handle-file-name-directory)
     (file-name-nondirectory . tramp-handle-file-name-nondirectory)
@@ -216,7 +216,11 @@ PRESERVE-UID-GID is completely ignored."
 
     (if tmpfile
 	;; Remote filename.
-	(rename-file tmpfile newname ok-if-already-exists)
+	(condition-case err
+	    (rename-file tmpfile newname ok-if-already-exists)
+	  ((error quit)
+	   (delete-file tmpfile)
+	   (signal (car err) (cdr err))))
 
       ;; Remote newname.
       (when (file-directory-p newname)
@@ -371,16 +375,18 @@ PRESERVE-UID-GID is completely ignored."
 (defun tramp-smb-handle-file-local-copy (filename)
   "Like `file-local-copy' for Tramp files."
   (with-parsed-tramp-file-name filename nil
+    (unless (file-exists-p filename)
+      (tramp-error
+       v 'file-error
+       "Cannot make local copy of non-existing file `%s'" filename))
     (let ((file (tramp-smb-get-localname localname t))
 	  (tmpfile (tramp-compat-make-temp-file filename)))
-      (unless (file-exists-p filename)
-	(tramp-error
-	 v 'file-error
-	 "Cannot make local copy of non-existing file `%s'" filename))
       (tramp-message v 4 "Fetching %s to tmp file %s..." filename tmpfile)
       (if (tramp-smb-send-command v (format "get \"%s\" %s" file tmpfile))
 	  (tramp-message
 	   v 4 "Fetching %s to tmp file %s...done" filename tmpfile)
+	;; Oops, an error.  We shall cleanup.
+	(delete-file tmpfile)
 	(tramp-error
 	 v 'file-error
 	 "Cannot make local copy of file `%s'" filename))
@@ -548,10 +554,14 @@ PRESERVE-UID-GID is completely ignored."
   (let ((tmpfile (file-local-copy filename)))
 
     (if tmpfile
-	;; remote filename
-	(rename-file tmpfile newname ok-if-already-exists)
+	;; Remote filename.
+	(condition-case err
+	    (rename-file tmpfile newname ok-if-already-exists)
+	  ((error quit)
+	   (delete-file tmpfile)
+	   (signal (car err) (cdr err))))
 
-      ;; remote newname
+      ;; Remote newname.
       (when (file-directory-p newname)
 	(setq newname (expand-file-name
 		      (file-name-nondirectory filename) newname)))
@@ -618,12 +628,13 @@ errors for shares like \"C$/\", which are common in Microsoft Windows."
 	 (list start end tmpfile append 'no-message lockname)))
 
       (tramp-message v 5 "Writing tmp file %s to file %s..." tmpfile filename)
-      (if (tramp-smb-send-command v (format "put %s \"%s\"" tmpfile file))
-	  (tramp-message
-	   v 5 "Writing tmp file %s to file %s...done" tmpfile filename)
-	(tramp-error v 'file-error "Cannot write `%s'" filename))
+      (unwind-protect
+	  (if (tramp-smb-send-command v (format "put %s \"%s\"" tmpfile file))
+	      (tramp-message
+	       v 5 "Writing tmp file %s to file %s...done" tmpfile filename)
+	    (tramp-error v 'file-error "Cannot write `%s'" filename))
+	(delete-file tmpfile))
 
-      (delete-file tmpfile)
       (unless (equal curbuf (current-buffer))
 	(tramp-error
 	 v 'file-error
@@ -905,20 +916,13 @@ connection if a previous connection has died for some reason."
 		  (executable-find tramp-smb-program))
 	  (error "Cannot find command %s in %s" tramp-smb-program exec-path))
 
-	(let* ((user (tramp-file-name-user vec))
-	       (host (tramp-file-name-host vec))
-	       (real-user user)
-	       (real-host host)
-	       domain port args)
-
-	  ;; Check for domain ("user%domain") and port ("host#port").
-	  (when (and user (string-match "\\(.+\\)%\\(.+\\)" user))
-	    (setq real-user (or (match-string 1 user) user)
-		  domain (match-string 2 user)))
-
-	  (when (and host (string-match "\\(.+\\)#\\(.+\\)" host))
-	    (setq real-host (or (match-string 1 host) host)
-		  port (match-string 2 host)))
+	(let* ((user      (tramp-file-name-user vec))
+	       (host      (tramp-file-name-host vec))
+	       (real-user (tramp-file-name-real-user vec))
+	       (real-host (tramp-file-name-real-host vec))
+	       (domain    (tramp-file-name-domain vec))
+	       (port      (tramp-file-name-port vec))
+	       args)
 
 	  (if share
 	      (setq args (list (concat "//" real-host "/" share)))
